@@ -56,6 +56,14 @@ namespace lsp
     {
         class Module;
         class ISchemaListener;
+        class IPlayListener;
+
+        enum import_flags_t
+        {
+            IMPORT_FLAG_NONE   = 0,
+            IMPORT_FLAG_PRESET = 1 << 0,
+            IMPORT_FLAG_PATCH  = 1 << 1
+        };
 
         /**
          * UI wrapper
@@ -85,8 +93,10 @@ namespace lsp
                 ui::Module                     *pUI;
                 resource::ILoader              *pLoader;            // Prefix-based resource loader
                 size_t                          nFlags;             // Flags
+                wssize_t                        nPlayPosition;      // Playback position of the current file preview
+                wssize_t                        nPlayLength;        // Overall playback file length in samples
                 expr::Variables                 sGlobalVars;        // Global variables
-                plug::position_t                sPosition;
+                plug::position_t                sPosition;          // Melodic position
 
                 lltl::parray<ui::IPort>         vPorts;             // All possible ports
                 lltl::parray<ui::IPort>         vSortedPorts;       // Alphabetically-sorted ports
@@ -97,6 +107,7 @@ namespace lsp
                 lltl::pphash<LSPString, LSPString> vAliases;        // Port aliases
                 lltl::parray<IKVTListener>      vKvtListeners;      // KVT listeners
                 lltl::parray<ISchemaListener>   vSchemaListeners;   // Schema change listeners
+                lltl::parray<IPlayListener>     vPlayListeners;     // List of playback listeners
 
             protected:
                 static ssize_t  compare_ports(const IPort *a, const IPort *b);
@@ -108,7 +119,6 @@ namespace lsp
                 void            build_global_config_header(LSPString *c);
                 status_t        init_visual_schema();
                 status_t        load_global_config(config::PullParser *parser);
-                status_t        import_settings(config::PullParser *parser, bool preset);
                 status_t        init_global_constants(const tk::StyleSheet *sheet);
                 status_t        apply_visual_schema(const tk::StyleSheet *sheet);
                 status_t        export_ports(config::Serializer *s, lltl::parray<IPort> *ports, const io::Path *relative);
@@ -120,6 +130,8 @@ namespace lsp
                 static void     drop_bundle_versions(lltl::pphash<LSPString, LSPString> *versions);
                 void            get_bundle_version_key(LSPString *key);
 
+                void            notify_play_position(wssize_t position, wssize_t length);
+
             protected:
                 static bool     set_port_value(ui::IPort *port, const config::param_t *param, size_t flags, const io::Path *base);
                 void            position_updated(const plug::position_t *pos);
@@ -286,23 +298,38 @@ namespace lsp
                 /**
                  * Export settings
                  * @param os output stream to perform export
-                 * @param relative the paths will be written relative to the passed path name, can be NULL
+                 * @param basedir the directory the config file will be written, can be NULL
                  * @return status of operation
                  */
-                virtual status_t            export_settings(io::IOutSequence *os, const char *relative);
-                virtual status_t            export_settings(io::IOutSequence *os, const LSPString *relative);
-                virtual status_t            export_settings(io::IOutSequence *os, const io::Path *relative = NULL);
+                virtual status_t            export_settings(io::IOutSequence *os, const char *basedir);
+                virtual status_t            export_settings(io::IOutSequence *os, const LSPString *basedir);
+                virtual status_t            export_settings(io::IOutSequence *os, const io::Path *basedir = NULL);
+
+                /**
+                 * Export settings
+                 * @param s output configuration serializer
+                 * @param basedir the directory the config file will be written, can be NULL
+                 * @return status of operation
+                 */
+                virtual status_t            export_settings(config::Serializer *s, const char *basedir);
+                virtual status_t            export_settings(config::Serializer *s, const LSPString *basedir);
+                virtual status_t            export_settings(config::Serializer *s, const io::Path *basedir = NULL);
 
                 /**
                  * Import settings
                  * @param the source (file name or input sequence)
-                 * @param preset the configuration is loaded as a preset
+                 * @param flags different flags (@see import_flags_t)
                  * @return status of operation
                  */
-                virtual status_t            import_settings(const char *file, bool preset);
-                virtual status_t            import_settings(const io::Path *file, bool preset);
-                virtual status_t            import_settings(const LSPString *file, bool preset);
-                virtual status_t            import_settings(io::IInSequence *is, bool preset);
+                virtual status_t            import_settings(const char *file, size_t flags);
+                virtual status_t            import_settings(const io::Path *file, size_t flags);
+                virtual status_t            import_settings(const LSPString *file, size_t flags);
+                virtual status_t            import_settings(io::IInSequence *is, size_t flags, const char *basedir);
+                virtual status_t            import_settings(io::IInSequence *is, size_t flags, const LSPString *basedir);
+                virtual status_t            import_settings(io::IInSequence *is, size_t flags, const io::Path *basedir = NULL);
+                virtual status_t            import_settings(config::PullParser *parser, size_t flags, const char *basedir);
+                virtual status_t            import_settings(config::PullParser *parser, size_t flags, const LSPString *basedir);
+                virtual status_t            import_settings(config::PullParser *parser, size_t flags, const io::Path *basedir = NULL);
 
                 /**
                  * Load visual schema for the wrapper
@@ -336,6 +363,32 @@ namespace lsp
                 virtual status_t            save_global_config(const io::Path *file);
                 virtual status_t            save_global_config(const LSPString *file);
 
+                /**
+                 * Send request to perform preview playback of the file
+                 *
+                 * @param file file name or empty/null if need to stop the playback
+                 * @param position initial position of playback in samples
+                 * @param release indicator for the stop event that informs that the sample won't
+                 *        be played anymore and the previously allocated memory should be released
+                 * @return status of operation
+                 */
+                virtual status_t            play_file(const char *file, wsize_t position, bool release);
+
+                /**
+                 * Subscribe for the listen update events, listener immediately receives
+                 * current playback position on the success subscription
+                 * @param listener the listener
+                 * @return status of the subscription
+                 */
+                virtual status_t            play_subscribe(IPlayListener *listener);
+
+                /**
+                 * Unsunscribe the listen update events
+                 * @param listener
+                 * @return status of operation
+                 */
+                virtual status_t            play_unsubscribe(IPlayListener *listener);
+
                 /**
                  * Add schema listener
                  * @param listener schema listener
