@@ -26,7 +26,7 @@
 
 #define LSP_PLUGINS_SAMPLER_VERSION_MAJOR                   1
 #define LSP_PLUGINS_SAMPLER_VERSION_MINOR                   0
-#define LSP_PLUGINS_SAMPLER_VERSION_MICRO                   6
+#define LSP_PLUGINS_SAMPLER_VERSION_MICRO                   8
 
 #define LSP_PLUGINS_SAMPLER_VERSION  \
     LSP_MODULE_VERSION( \
@@ -142,13 +142,42 @@ namespace lsp
         static const port_item_t sampler_x48_mixer_lines[] =
         {
             { "Instruments", "sampler.instruments" },
-            { "Mixer 0-11", "sampler.mixer_0:11" },
+            { "Mixer 0-11",  "sampler.mixer_0:11" },
             { "Mixer 12-23", "sampler.mixer_12:23" },
             { "Mixer 24-35", "sampler.mixer_24:35" },
             { "Mixer 36-47", "sampler.mixer_36:47" },
             { NULL, NULL }
         };
 
+        static const port_item_t sampler_crossfade_type[] =
+        {
+            { "Linear",         "fade.linear"      },
+            { "Const Power",    "fade.const_power" },
+            { NULL, NULL }
+        };
+
+        static const port_item_t sampler_loop_mode[] =
+        {
+            { "Simple: Direct",     "sampler.loop.simple.direct"            },
+            { "Simple: Reverse",    "sampler.loop.simple.reverse"           },
+            { "PP: Half Direct",    "sampler.loop.ping_pong.direct_half"    },
+            { "PP: Half Reverse",   "sampler.loop.ping_pong.reverse_half"   },
+            { "PP: Full Direct",    "sampler.loop.ping_pong.direct_full"    },
+            { "PP: Full Reverse",   "sampler.loop.ping_pong.reverse_full"   },
+            { "PP: Smart Direct",   "sampler.loop.ping_pong.direct_smart"   },
+            { "PP: Smart Reverse",  "sampler.loop.ping_pong.reverse_smart"  },
+            { NULL, NULL }
+        };
+
+        static const port_item_t sampler_sample_editor_tabs[] =
+        {
+            { "Main",           "sampler.edit.main"      },
+            { "Pitch",          "sampler.edit.pitch"     },
+            { "Stretch",        "sampler.edit.stretch"   },
+            { "Loop",           "sampler.edit.loop"      },
+            { NULL, NULL }
+        };
+
         #define S_DO_GROUP_PORTS(i) \
             STEREO_PORT_GROUP_PORTS(dout_ ## i, "dol_" #i, "dor_" #i)
 
@@ -169,7 +198,8 @@ namespace lsp
             CONTROL("fout", "Note-off fadeout", U_MSEC, sampler_metadata::FADEOUT), \
             DRY_GAIN(1.0f),         \
             WET_GAIN(1.0f),         \
-            OUT_GAIN
+            OUT_GAIN, \
+            COMBO("sets", "Sample Editor Tab Selection", 0, sampler_sample_editor_tabs)
 
         #define S_DO_CONTROL \
             SWITCH("do_gain", "Apply gain to direct-out", 1.0f), \
@@ -178,6 +208,19 @@ namespace lsp
         #define S_SAMPLE_FILE(gain)        \
             PATH("sf", "Sample file"), \
             CONTROL("pi", "Sample pitch", U_SEMITONES, sampler_metadata::SAMPLE_PITCH), \
+            SWITCH("so", "Sample stretch enabled", 0.0f), \
+            CONTROL("st", "Sample relative stretch time", U_MSEC, sampler_metadata::SAMPLE_STRETCH), \
+            CONTROL("ss", "Sample stretch region start", U_MSEC, sampler_metadata::SAMPLE_LENGTH), \
+            CONTROL("se", "Sample stretch region end", U_MSEC, sampler_metadata::SAMPLE_LENGTH), \
+            CONTROL("sc", "Sample stretch chunk", U_MSEC, sampler_metadata::SAMPLE_STRETCH_CHUNK), \
+            CONTROL("sx", "Sample stretch fade", U_PERCENT, sampler_metadata::SAMPLE_STRETCH_FADE), \
+            COMBO("xt", "Sample stretch crossfade type", 1, sampler_crossfade_type), \
+            SWITCH("lo", "Sample loop enabled", 0.0f), \
+            COMBO("lm", "Sample loop mode", 0, sampler_loop_mode), \
+            CONTROL("lb", "Sample loop region start", U_MSEC, sampler_metadata::SAMPLE_LENGTH), \
+            CONTROL("le", "Sample loop region end", U_MSEC, sampler_metadata::SAMPLE_LENGTH), \
+            CONTROL("ll", "Sample loop fade", U_MSEC, sampler_metadata::SAMPLE_LENGTH), \
+            COMBO("lx", "Sample loop crossfade type", 1, sampler_crossfade_type), \
             CONTROL("hc", "Sample head cut", U_MSEC, sampler_metadata::SAMPLE_LENGTH), \
             CONTROL("tc", "Sample tail cut", U_MSEC, sampler_metadata::SAMPLE_LENGTH), \
             CONTROL("fi", "Sample fade in", U_MSEC, sampler_metadata::SAMPLE_LENGTH), \
@@ -189,10 +232,17 @@ namespace lsp
             SWITCH("on", "Sample enabled", 1.0f), \
             TRIGGER("ls", "Sample listen"), \
             SWITCH("rs", "Sample reverse", 0.0f), \
+            SWITCH("pc", "Sample auto-compensate", 0.0f), \
+            CONTROL("xx", "Sample auto-compensate fade", U_PERCENT, sampler_metadata::SAMPLE_COMPENSATE_FADE), \
+            CONTROL("cc", "Sample auto-compensate stretch chunk", U_MSEC, sampler_metadata::SAMPLE_COMPENSATE_CHUNK), \
+            COMBO("xc", "Sample auto-compensate crossfade type", 1, sampler_crossfade_type), \
             gain, \
             BLINK("ac", "Sample activity"), \
+            METER("pp", "Sample play position", U_MSEC, sampler_metadata::SAMPLE_PLAYBACK), \
             BLINK("no", "Sample note on event"), \
-            { "fl", "Sample length", U_MSEC, R_METER, F_OUT | F_LOWER | F_UPPER | F_STEP, \
+            { "fl", "Length of loaded sample", U_MSEC, R_METER, F_OUT | F_LOWER | F_UPPER | F_STEP, \
+                    sampler_metadata::SAMPLE_LENGTH_MIN, sampler_metadata::SAMPLE_LENGTH_MAX, 0, sampler_metadata::SAMPLE_LENGTH_STEP, NULL }, \
+            { "al", "Actual length of loaded sample", U_MSEC, R_METER, F_OUT | F_LOWER | F_UPPER | F_STEP, \
                     sampler_metadata::SAMPLE_LENGTH_MIN, sampler_metadata::SAMPLE_LENGTH_MAX, 0, sampler_metadata::SAMPLE_LENGTH_STEP, NULL }, \
             STATUS("fs", "Sample load status"), \
             MESH("fd", "Sample file contents", sampler_metadata::TRACKS_MAX, sampler_metadata::MESH_SIZE)
@@ -708,7 +758,7 @@ namespace lsp
             NULL,
             LSP_PLUGINS_SAMPLER_VERSION,
             sampler_classes,
-            E_DUMP_STATE,
+            E_DUMP_STATE | E_FILE_PREVIEW,
             sampler_mono_ports,
             "sampling/single/mono.xml",
             NULL,
@@ -730,7 +780,7 @@ namespace lsp
             NULL,
             LSP_PLUGINS_SAMPLER_VERSION,
             sampler_classes,
-            E_DUMP_STATE,
+            E_DUMP_STATE | E_FILE_PREVIEW,
             sampler_stereo_ports,
             "sampling/single/stereo.xml",
             NULL,
@@ -752,7 +802,7 @@ namespace lsp
             NULL,
             LSP_PLUGINS_SAMPLER_VERSION,
             sampler_classes,
-            E_DUMP_STATE | E_KVT_SYNC,
+            E_DUMP_STATE | E_KVT_SYNC | E_FILE_PREVIEW,
             sampler_x12_ports,
             "sampling/multiple.xml",
             NULL,
@@ -774,7 +824,7 @@ namespace lsp
             NULL,
             LSP_PLUGINS_SAMPLER_VERSION,
             sampler_classes,
-            E_DUMP_STATE | E_KVT_SYNC,
+            E_DUMP_STATE | E_KVT_SYNC | E_FILE_PREVIEW,
             sampler_x24_ports,
             "sampling/multiple.xml",
             NULL,
@@ -796,7 +846,7 @@ namespace lsp
             NULL,
             LSP_PLUGINS_SAMPLER_VERSION,
             sampler_classes,
-            E_DUMP_STATE | E_KVT_SYNC,
+            E_DUMP_STATE | E_KVT_SYNC | E_FILE_PREVIEW,
             sampler_x48_ports,
             "sampling/multiple.xml",
             NULL,
@@ -818,7 +868,7 @@ namespace lsp
             NULL,
             LSP_PLUGINS_SAMPLER_VERSION,
             sampler_classes,
-            E_DUMP_STATE | E_KVT_SYNC,
+            E_DUMP_STATE | E_KVT_SYNC | E_FILE_PREVIEW,
             sampler_x12_do_ports,
             "sampling/multiple.xml",
             NULL,
@@ -840,7 +890,7 @@ namespace lsp
             NULL,
             LSP_PLUGINS_SAMPLER_VERSION,
             sampler_classes,
-            E_DUMP_STATE | E_KVT_SYNC,
+            E_DUMP_STATE | E_KVT_SYNC | E_FILE_PREVIEW,
             sampler_x24_do_ports,
             "sampling/multiple.xml",
             NULL,
@@ -862,7 +912,7 @@ namespace lsp
             NULL,
             LSP_PLUGINS_SAMPLER_VERSION,
             sampler_classes,
-            E_DUMP_STATE | E_KVT_SYNC,
+            E_DUMP_STATE | E_KVT_SYNC | E_FILE_PREVIEW,
             sampler_x48_do_ports,
             "sampling/multiple.xml",
             NULL,
