@@ -125,9 +125,10 @@ namespace lsp
                 // Focus operations
                 inline bool         check_focus(Widget *w) const    { return pFocused == w; }
                 virtual bool        take_focus(Widget *w);
-                bool                kill_focus(Widget *w);
+                bool                do_kill_focus(Widget *w);
                 size_t              make_key_pressed(ws::code_t code);
                 size_t              make_key_released(ws::code_t code);
+                status_t            init_internal(bool create_handle);
 
             protected:
                 virtual Widget     *find_widget(ssize_t x, ssize_t y);
@@ -233,8 +234,6 @@ namespace lsp
 
                 virtual bool            take_focus();
 
-                virtual bool            kill_focus();
-
                 virtual bool            has_parent() const;
 
             public:
